"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const express = require("express");
const cors = require("cors");
const bodyParser = require("body-parser");
const uuid = require("uuid");
const Debug = require("debug");
const debug = Debug("apifs:gateway");
const pjson = require("../package.json");
const RequestHandler_1 = require("./RequestHandler");
const CircuitClient_1 = require("./CircuitClient");
const ConfigValidator_1 = require("./ConfigValidator");
exports.AUTH_TYPES = {
    NONE: "NONE",
    USER: "USER",
    MACHINE: "MACHINE",
    USER_AND_MACHINE: "USER_AND_MACHINE",
};
exports.REQUEST_METHODS = {
    GET: "GET",
    POST: "POST",
    PUT: "PUT",
    PATCH: "PATCH",
    DELETE: "DELETE",
    HEAD: "HEAD",
    OPTIONS: "OPTIONS",
};
exports.CORRELATION_ID_HEADER = "correlation-id";
const WARNING_LOG_MS = 3000;
exports.DEFAULT_TIMEOUT = 1500;
exports.SECRET_HEADER = "apifs-secret";
exports.BAD_PROXY_HEADERS = [
    "content-encoding",
    "content-language",
    "content-length",
    "content-location",
    "content-md5",
    "content-range",
    "connection",
    "date",
    "expect",
    "max-forwards",
    "pragma",
    "proxy-authorization",
    "te",
    "transfer-encoding",
    "via",
];
class Gateway {
    constructor(gatewayConfig, functionsConfig) {
        try {
            new ConfigValidator_1.default(gatewayConfig, functionsConfig)
                .validate();
            debug("Configuration is valid.");
        }
        catch (error) {
            debug("Configuration Invalid:", error.message);
            process.exit(1);
        }
        this.gatewayConfig = gatewayConfig;
        this.functionsConfig = functionsConfig;
        this.app = express();
        this.circuitClient = new CircuitClient_1.default(this);
        this.requestHandler = new RequestHandler_1.default(this);
        this.alive = true;
        this.ready = false;
    }
    isAlive() {
        return this.alive;
    }
    isReady() {
        return this.ready;
    }
    getHealthStatus() {
        const alive = this.isAlive();
        const status = {
            status: alive ? "UP" : "DOWN",
        };
        return [
            alive ? 200 : 503,
            status,
        ];
    }
    init() {
        debug("Init..");
        if (this.gatewayConfig.enableWCCors) {
            this.app.use(cors());
        }
        // NOTE: this is slow, we should do this ourselves and pipe
        // however, its fast enough for now ;)
        this.app.use(bodyParser.json());
        this.app.get("/admin/healthcheck", (req, res) => {
            res.status(this.isAlive() ? 200 : 503).end();
        });
        this.app.get("/admin/health", (req, res) => {
            const [status, body] = this.getHealthStatus();
            res.status(status).json(body);
        });
        this.app.get("/admin/ready", (req, res) => {
            res.status(this.isReady() ? 200 : 503).end();
        });
        this.app.use((req, res, _) => {
            const onEndOfRequest = () => {
                res.removeListener("finish", onEndOfRequest);
                res.removeListener("close", onEndOfRequest);
                const diff = Date.now() - res.locals.startTime;
                debug(`Access-Log: ${req.method} : ${req.url} => ${res.statusCode} after ${diff} ms.`);
                if (diff >= WARNING_LOG_MS) {
                    debug(`Slow request alert: ${req.method} : ${req.url} took ${diff} ms.`);
                }
            };
            if (this.gatewayConfig.enableCorrelationIds &&
                !req.headers[exports.CORRELATION_ID_HEADER]) {
                req.headers[exports.CORRELATION_ID_HEADER] = uuid.v4();
            }
            res.set("x-powered-by", `${pjson.name}/${pjson.version}`);
            res.set("cache-control", "no-cache");
            // crawler check
            if (req.path === "/robots.txt") {
                res.status(200);
                res.set("content-type", "text/plain");
                return res.end("User-agent: *\nDisallow: /");
            }
            // dev browser check
            if (req.path === "/favicon.ico") {
                return res.status(404).end();
            }
            res.on("finish", onEndOfRequest);
            res.on("close", onEndOfRequest);
            res.locals.startTime = Date.now();
            this.requestHandler
                .handle(req, res)
                .catch((error) => {
                debug("An error occured during request handle:", error.message, error.stack);
                res.status(500).json({
                    error: error.message,
                });
            });
        });
        return this;
    }
    run() {
        debug("Starting..");
        return new Promise((resolve, reject) => {
            this.server = this.app.listen(this.gatewayConfig.port, (error) => {
                if (error) {
                    return reject(error);
                }
                debug("Started, listening on port", this.gatewayConfig.port);
                this.ready = true;
                resolve(this);
            });
        });
    }
    close() {
        debug("Closing..");
        if (this.server) {
            return new Promise((resolve, reject) => {
                this.server.close((error) => {
                    if (error) {
                        return reject(error);
                    }
                    resolve(this);
                });
            });
        }
        else {
            return Promise.reject(new Error("No server initialised."));
        }
    }
}
exports.default = Gateway;
